#!/usr/bin/env python
#*****************************************************************************#
#**
#**  Local Linear Projection
#**
#**    Kyle D. Feuz, 2013
#**
#** Tools by the Center for Advanced Studies in Adaptive Systems at
#**  the School of Electrical Engineering and Computer Science at
#**  Washington State University
#** 
#** Copyright Washington State University, 2013
#** Copyright Kyle D. Feuz, 2013
#** 
#** Licenced under the Creative Commons (http://creativecommons.org):
#** Attribution-NonCommercial-ShareAlike 3.0 Unported
#** 
#** If this code is used for public research, any resulting publications need
#** to cite work done by Kyle D. Feuz at the Center for Advanced Study of 
#** Adaptive Systems (CASAS) at Washington State University.
#** 
#** Contact: Kyle D. Feuz (kyle.feuz@wsu.edu)
#** Contact: Diane J. Cook (cook@eecs.wsu.edu)
#*****************************************************************************#
from Orange.projection.linear import Projector, Pca 
from Orange.classification.knn import FindNearestConstructor
from Orange.data import Table
import scipy.linalg as LA
from scipy.spatial import distance_matrix
import numpy as np

USE_KNN = True

class Lpp(Pca):

   #def __new__(cls, dataset=None, **kwds):
   #   optimizer = Pca.__new__(cls)

   #   if dataset is not None:
   #      optimizer.__init__(**kwds)
   #      return optimizer(dataset)
   #   else:
   #      return optimizer

   def __init__(self, k=5, **kwds):
      Pca.__init__(self, **kwds)
      self.k = k
   
   def __call__(self, dataset):

      Xd, stdev, mean, relevant_features = self._normalize_data(dataset)
      table = Table(dataset.domain, Xd)
      if USE_KNN:
         W = self._construct_w(table)
      else:
         W = self._construct_w(Xd)
      D = np.identity(w.shape[0])*np.sum(w, 0)
      L = D-W


      components, variances = self._perform_lpp(dataset, Xd, Xg)
      variances, components, variance_sum = self._select_components(variances, components)

      return LppProjector(input_domain=dataset.domain,
                          center=mean,
                          scale=stdev,
                          standardize=self.standardize,
                          projection=components,
                          variances=variances,
                          variance_sum=variance_sum)

   def _construct_w(self, Xd):
      if USE_KNN:
         ncc = FindNearestConstructor()
         #TODO Finish this function
      else:
         w = distance_matrix(Xd, Xd)
         ind = np.argsort(w)[:, self.k+1]
         for i,v in enumerate(ind):
            mask = w[i] <= w[i][v]
            w[i][mask] = 1
            w[i][~mask] = 0
         w = w + w.T
         w[w>1] = 1
         np.fill_diagonal(w, 0)
      return w
      



   def _perform_lpp(self, Xd, L, D):
      A = np.dot(np.dot(Xd.T, L), Xd)
      B = np.dot(np.dot(Xd.T, D), Xd)
      eigvals, eigvecs = LA.eig(A, B)
      ind = np.argsort(eigvals)
      eigvals = eigvals[ind]
      eigvecs = eigvecs[:, ind]
      return eigvecs.T, eigvals

class LppProjector(Projector):

   pass
